<?php

namespace Fastrack\CalendarEvents;

use Fastrack\Utilities\Slug;
use SilverStripe\Control\Controller;
use SilverStripe\Core\Environment;
use SilverStripe\ORM\DataObject;
use SilverStripe\Forms\ToggleCompositeField;
use SilverStripe\Control\Director;
use SilverStripe\ORM\ArrayList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;
use SilverStripe\View\ArrayData;
use SilverStripe\Forms\CheckboxField;
use SilverStripe\Forms\RequiredFields;
use Fastrack\Gallery\Gallery;
use SilverStripe\View\Requirements;

class CalendarEvent extends DataObject implements PermissionProvider
{

	private static $table_name = 'calendar_event';

	private static $db = [
		'Name' => 'Varchar(255)',
		'Website' => 'Varchar(255)',
		'Active' => 'Boolean(true)',
		'Featured' => 'Boolean',
		'Price' => 'Currency',
		'Venue' => 'Varchar(255)',
		'StartAt' => 'Date',
		'FinishAt' => 'Date',
		'RepeatsWeekly' => 'Boolean',
		'RepeatsSunday' => 'Boolean',
		'RepeatsMonday' => 'Boolean',
		'RepeatsTuesday' => 'Boolean',
		'RepeatsWednesday' => 'Boolean',
		'RepeatsThursday' => 'Boolean',
		'RepeatsFriday' => 'Boolean',
		'RepeatsSaturday' => 'Boolean',
	];

	private static $has_many = [
		'Galleries' => Gallery::class,
	];

	private static $many_many = [
		'Categories' => CalendarEventCategory::class,
	];

	private static $translate = [
		'Name',
	];

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		$fields->removeByName([
			'RepeatsSunday',
			'RepeatsMonday',
			'RepeatsTuesday',
			'RepeatsWednesday',
			'RepeatsThursday',
			'RepeatsFriday',
			'RepeatsSaturday'
		]);
		$fields->addFieldToTab('Root.Main', ToggleCompositeField::create('RepeatSettings', 'Repeat Settings', [
			CheckboxField::create('RepeatsSunday', 'Repeats Sunday'),
			CheckboxField::create('RepeatsMonday', 'Repeats Monday'),
			CheckboxField::create('RepeatsTuesday', 'Repeats Tuesday'),
			CheckboxField::create('RepeatsWednesday', 'Repeats Wednesday'),
			CheckboxField::create('RepeatsThursday', 'Repeats Thursday'),
			CheckboxField::create('RepeatsFriday', 'Repeats Friday'),
			CheckboxField::create('RepeatsSaturday', 'Repeats Saturday')
		]));
		return $fields;
	}

	public function getCMSValidator()
	{
		return new RequiredFields(['Name']);
	}

	public function isValid()
	{
		if ($this->FinishAt && $this->obj('FinishAt')->InPast()) return false;
		if (!$this->FinishAt && $this->obj('StartAt')->InPast()) return false;
		if (!$this->Active) return false;
		return true;
	}

	protected function onBeforeWrite()
	{
		$this->Slug = Slug::generateSlug($this->Name, $this->Slug);
		Gallery::createDefaultGalleries($this);
		parent::onBeforeWrite();
	}

	protected function onAfterWrite()
	{
		$event = CalendarEvent::get()->byID($this->ID);
		parent::onAfterWrite();
	}

	public function Link()
	{
		$link = new ArrayList([]);
		$eventsPage = CalendarEventsPage::get()->first();
		if ($eventsPage) {
			$link->push($eventsPage->Link());
		}
		$link->push($this->Slug);
		$this->extend('updateLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function AbsoluteLink()
	{
		return Director::absoluteURL($this->Link());
	}

	public function getRepeatedDays()
	{
		$daysOfTheWeek = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
		$days = [];
		foreach ($daysOfTheWeek as $day) {
			$repeatName = 'Repeats' . $day;
			if ($this->$repeatName) {
				$days[] = ArrayData::create(['Day' => $day]);
			}
		}
		return new ArrayList($days);
	}

	public function providePermissions()
	{
		$permisions = [
			'MANAGE_CALENDAR_EVENTS' => [
				'name' => 'Manage calendar events',
				'category' => 'Calendar events',
			],
		];
		$this->extend('updatePermisions', $permisions);
		return $permisions;
	}

	public function canView($member = null)
	{
		if (!$member) {
			if (!Security::getCurrentUser()) {
				return $this->Active;
			}
		}
		return Permission::check(['MANAGE_CALENDAR_EVENTS']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_CALENDAR_EVENTS']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_CALENDAR_EVENTS']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_CALENDAR_EVENTS']);
	}
}
